package org.svaor.preferences;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import static org.apache.commons.lang.ArrayUtils.EMPTY_STRING_ARRAY;

/** @author Smelik Evgeniy */
public class PreferencesServiceImpl implements PreferencesService {
	private PreferencesProvider preferencesProvider;

	public PreferencesServiceImpl() {
	}

	@NotNull
	@Override
	public Long loadOrCreateRootNode(@Nullable String username) {
		return preferencesProvider.loadOrCreateRootNode(username);
	}

	@NotNull
	@Override
	public Long[] setProperty(@NotNull Long rootNodeId, @Nullable String[] nodeNames, @NotNull String nodePath, @NotNull String key, @NotNull String value) {
		return preferencesProvider.setProperty(rootNodeId, nodeNames, key, value);
	}

	@Nullable
	@Override
	public String getProperty(@NotNull String nodePath, @Nullable Long nodeId, @NotNull String key) {
		//nodeId может оказаться null в случае если node еще не был сохранен в БД, но мы пытаемся получить из него пропертю
		if (nodeId == null) {
			return null;
		}
		return preferencesProvider.getProperty(nodeId, key);
	}

	@Override
	public void removeProperty(@NotNull String nodePath, @NotNull Long nodeId, @NotNull String key) {
		preferencesProvider.removeProperty(nodeId, key);
	}

	@Override
	public void removeNode(@NotNull String parentNodePath, @NotNull String nodeName, @NotNull Long nodeId) {
		preferencesProvider.removeNode(nodeId);
	}

	@NotNull
	@Override
	public String[] getPropertyKeys(@NotNull String nodePath, @Nullable Long nodeId) {
		if (nodeId == null) {
			return EMPTY_STRING_ARRAY;
		}
		return preferencesProvider.getPropertyKeys(nodeId);
	}

	@NotNull
	@Override
	public String[] getNodeNames(@NotNull String nodePath, @Nullable Long nodeId) {
		if (nodeId == null) {
			return EMPTY_STRING_ARRAY;
		}
		return preferencesProvider.getNodeNames(nodeId);
	}

	@Nullable
	@Override
	public Long loadNode(@NotNull String parentNodePath, @NotNull Long parentNodeId, @NotNull String childNodeName) {
		return preferencesProvider.loadNode(parentNodeId, childNodeName);
	}

	public void setPreferencesProvider(PreferencesProvider preferencesProvider) {
		this.preferencesProvider = preferencesProvider;
	}
}
