package org.svaor.preferences.model;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.Serializable;
import java.util.Date;
import java.util.Map;

/**
 * Preferences node
 * @author Smelik Evgeniy
 */
public interface Node extends Serializable, Cloneable {
	long getId();

	@NotNull
	String getName();

	/** @return node owner user name. The value may be null if current node is a system node */
	@Nullable
	String getUsername();

	@NotNull
	Date getCreationDate();

	/** @return parent node. The value may be null if current node is root node */
	@Nullable
	Node getParent();

	/** @param parentNode parent node. The value may be null if the current node is root node */
	void setParent(@Nullable Node parentNode);

	/** @return child nodes map. This map key is a child node name */
	Map<String, Node> getChildNodes();

	/** @return properties map. This map key is a property name */
	Map<String, Property> getProperties();
}
