package org.svaor.preferences.provider.model;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.svaor.preferences.model.Node;
import org.svaor.preferences.model.Property;

import java.util.Date;
import java.util.Map;

import static com.google.common.collect.Maps.newHashMap;
import static org.apache.commons.lang.builder.ToStringStyle.SHORT_PREFIX_STYLE;

/** @author Smelik Evgeniy */
public class NodeImpl implements Node {
	public static final String ROOT_NODE_NAME = "root";

	@SuppressWarnings({"UnusedDeclaration"})
	private Long id;
	@NotNull
	private String name;
	@Nullable
	private String username;
	@NotNull
	private Date created = new Date();
	@Nullable
	private Node parent;
	private Map<String, Node> childNodes = newHashMap();
	private Map<String, Property> properties = newHashMap();

	public NodeImpl() {
	}

	public NodeImpl(@NotNull String name) {
		this.name = name;
	}

	public NodeImpl(@NotNull String name, @Nullable Node parent) {
		this(name);

		setParent(parent);
	}

	@Override
	public long getId() {
		return id;
	}

	@NotNull
	@Override
	public String getName() {
		return name;
	}

	public void setName(@NotNull String name) {
		this.name = name;
	}

	@Nullable
	public String getUsername() {
		return username;
	}

	public void setUsername(@Nullable String username) {
		this.username = username;
	}

	@NotNull
	@Override
	public Date getCreationDate() {
		return created;
	}

	@Nullable
	@Override
	public Node getParent() {
		return parent;
	}

	public void setParent(@Nullable Node parent) {
		this.parent = parent;
		if (parent != null) {
			this.username = parent.getUsername();
		}

		refreshCacheOfChildNodes(parent);
	}

	private void refreshCacheOfChildNodes(Node parent) {
		if (parent == null) {
			return;
		}

		//no superfluous updates here
		parent.getChildNodes().put(name, this);
	}

	@Override
	public Map<String, Node> getChildNodes() {
		return childNodes;
	}

	public void setChildNodes(Map<String, Node> childNodes) {
		this.childNodes = childNodes;
	}

	@Override
	public Map<String, Property> getProperties() {
		return properties;
	}

	public void setProperties(Map<String, Property> properties) {
		this.properties = properties;
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this, SHORT_PREFIX_STYLE).
				append("id", id).
				append("name", name).
				append("username", username).
				toString();
	}
}
