package org.svaor.preferences.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.util.Assert;

/**
 * This class is eager initialized by spring IoC container and provides a link to current root spring context.
 * @author Smelik Evgeniy
 */
public class ApplicationContextProvider implements ApplicationContextAware {
	private final Logger log = LoggerFactory.getLogger(ApplicationContextProvider.class);

	private static ApplicationContext applicationContext;

	private ApplicationContextProvider() {
	}

	public static ApplicationContext getApplicationContext() {
		Assert.state(applicationContext != null, "Application context holder is not initialized");

		return applicationContext;
	}

	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		if (ApplicationContextProvider.applicationContext != null) {
			log.warn("Application context holder was already initialized at least one time");
			return;
		}

		ApplicationContextProvider.applicationContext = getRootContext(applicationContext);
	}

	private ApplicationContext getRootContext(ApplicationContext applicationContext) {
		while (applicationContext.getParent() != null) {
			applicationContext = applicationContext.getParent();
		}
		return applicationContext;
	}
}
