package org.svaor.preferences;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import org.apache.commons.lang.reflect.FieldUtils;
import org.junit.After;
import org.junit.runner.RunWith;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import java.lang.reflect.Field;

import static org.svaor.preferences.provider.PreferencesProviderImpl.CACHE_USERNAME_ROOT_NODE_NAME;

/**
 * Abstract test for database preferences API implementation.
 * @author Smelik Evgeniy
 */
@ContextConfiguration(locations = {"classpath:/org/svaor/preferences/provider/test-preferences-context.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public abstract class AbstractPreferencesLowLevelTest {
	private final Logger log = LoggerFactory.getLogger(AbstractPreferencesLowLevelTest.class);

	@Autowired
	@Qualifier("preferencesHibernateTemplate")
	protected HibernateTemplate hibernateTemplate;
	@Autowired
	@Qualifier("preferencesTransactionTemplate")
	protected TransactionTemplate transactionTemplate;

	public AbstractPreferencesLowLevelTest() {
	}

	@After
	public void clear() throws Exception {
		clearTables();
		clearCache();
	}

	protected void clearTables() {
		log.debug("Clearing database tables");
		transactionTemplate.execute(new TransactionCallbackWithoutResult() {
			@Override
			protected void doInTransactionWithoutResult(TransactionStatus status) {
				int updateResult = hibernateTemplate.bulkUpdate("DELETE FROM PropertyImpl");
				if (log.isDebugEnabled()) {
					log.debug("Deleted [{}] properties", updateResult);
				}

				updateResult = hibernateTemplate.bulkUpdate("DELETE FROM NodeImpl");
				if (log.isDebugEnabled()) {
					log.debug("Deleted [{}] nodes", updateResult);
				}
			}
		});
	}

	protected void clearCache() throws IllegalAccessException {
		final CacheManager cacheManager = CacheManager.getInstance();
		final Cache cache = cacheManager.getCache(CACHE_USERNAME_ROOT_NODE_NAME);
		final int size = cache.getSize();
		cache.removeAll();

		log.debug("Deleted [{}] root nodes from cache", size);

		final Field fieldSystemRoot = FieldUtils.getDeclaredField(PreferencesImpl.class, "systemRoot", true);
		final PreferencesImpl systemRoot = (PreferencesImpl) fieldSystemRoot.get(null);
		if (systemRoot != null) {
			fieldSystemRoot.set(null, null);
		}

		if (log.isDebugEnabled()) {
			log.debug("PreferencesImpl's systemRoot is cleared. Previous value was [{}]", systemRoot);
		}
	}
}
